@@ -11,10 +11,3 @@ def MotherNaming(s,pref=None):
     l=s.split("_")
     l=["%s%s" % (i[0].upper(),i[1:]) for i in l]
     return (pref or "") + "".join(l)
-
-def somethingFromFile(f, *args):
-    d= {}
-    execfile(f, d)
-    res= [d[k] for k in args]
-    return tuple(res)
-
