@@ -325,19 +325,22 @@ class PsygresFly(Speaker):
                 OKI_COL(self.session_name))
 
         if not hasattr(self, '_pool_queue'):
+            self.log_insane("Session %s dies here.", 
+                    OKI_COL(self.session_name))
             self.pg_conn.commit()
             return
 
-        try:
-            self.pg_conn.commit()
+        res= self.commit()
 
-        except Exception, ss:
-            PsygresPool.orphan()
+        if res<>0:
+            PsygresPool.discard(self)
             self.log_int_raise(
-                    "Unable to commit queries: %s", ERR_COL(ss))
+                    "Session %s, unable to commit queries(): %s. "\
+                    "Broken session removd from the pool.",
+                    ERR_COL(self.session_name), ERR_COL(res))
 
-        PsygresPool.back_home(self)
-        self.log_info(
+        PsygresPool.backHome(self)
+        self.log_debug(
                 "Session %s back to the Pool.", 
                 OKI_COL(self.session_name))
 
@@ -347,65 +350,92 @@ class PsygresFly(Speaker):
         self.pg_conn.rollback()
 
     def commit(self):
-        self.log_noise(
-                "commit() ignored inside Session (%s): "
-                "queries will be committed with session.endSession()", 
-                INF_COL(self.session_name))
+        try:
+            self.pg_conn.commit()
+            self.log_debug("Queries Committed for Session %s.", 
+                    OKI_COL(self.session_name))
+            return 0
+        except Exception, ss:
+            return ss
 
 
-class PsygresPool(Speaker):
+class PsygresPool:
 
     import threading
-    from Queue import Queue, Empty
+    from Queue import Empty as _empty_queue
 
     _pool_initialized= False
 
-    pool_max= 10
-    pool_min= 4
-    pool_timeout= 15
-    pool_current= 0
-    pool_queue= None
+    _pool_max= 10
+    _pool_min= 4
+    _pool_timeout= 15
+    _pool_current= 0
+    _pool_orphans= {}
+    _pool_queue= None
 
-    pool_current_mutex= threading.Lock()
-    pool_get_mutex= threading.Lock()
+    _pool_current_mutex= threading.Lock()
+    _pool_get_mutex= threading.Lock()
+    _pool_orphans_mutex= threading.Lock()
+
+    del threading
+
+    @staticmethod
+    def _add_orphan(sname, db):
+        m= PsygresPool._pool_orphans_mutex
+        m.acquire()
+        PsygresPool._pool_orphans[id(db)]= sname
+        m.release()
+
+    @staticmethod
+    def _del_orphan(db):
+        m= PsygresPool._pool_orphans_mutex
+        m.acquire()
+        PsygresPool._pool_orphans.pop(id(db))
+        m.release()
 
     @staticmethod
     def _full():
-        m= PsygresPool.pool_current_mutex
+        m= PsygresPool._pool_current_mutex
         m.acquire()
-        res= PsygresPool.pool_current== PsygresPool.pool_max
+        res= PsygresPool._pool_current== PsygresPool._pool_max
         m.release()
         return res
 
     @staticmethod
-    def session_number():
-        mc= PsygresPool.pool_current_mutex
-        mg= PsygresPool.pool_get_mutex
+    def status():
+        """ status() -> (available, total, min, max, orphaned)
+        """
+        mc= PsygresPool._pool_current_mutex
+        mg= PsygresPool._pool_get_mutex
+        mo= PsygresPool._pool_orphans_mutex
         mc.acquire()
         mg.acquire()
-        total= PsygresPool.pool_current
-        available= PsygresPool.pool_queue.qsize()
+        mo.acquire()
+        total= PsygresPool._pool_current
+        available= PsygresPool._pool_queue.qsize()
+        orphaned= PsygresPool._pool_orphans.values()
+        mo.release()
         mg.release()
         mc.release()
-        max= PsygresPool.pool_max
-        min= PsygresPool.pool_min
-        return (available, total, min, max)
+        max= PsygresPool._pool_max
+        min= PsygresPool._pool_min
+        return (available, total, min, max, orphaned)
 
     @staticmethod
     def _add_conn(n=1, pg_conn=None):
-        """ Never call me directly: use addConnection!"""
-        max= PsygresPool.pool_max
-        cur= PsygresPool.pool_current
+        """ Never call me directly: use _addConnection!"""
+        max= PsygresPool._pool_max
+        cur= PsygresPool._pool_current
         dif= max- cur
 
         if not dif:
-            PsygresPool.log_warning(
+            Speaker.log_warning(
                     "No space left in PsygresPool "
                     "(max= %s)", ERR_COL(max))
             return dif
 
         elif dif< n:
-            PsygresPool.log_warning(
+            Speaker.log_warning(
                     "Adding only %s new connection(s) "
                     "(max= %s, cur= %s, req= %s)",
                     OKI_COL(dif), ERR_COL(max), 
@@ -413,23 +443,32 @@ class PsygresPool(Speaker):
 
         else:
             dif= n
-            PsygresPool.log_info(
+            Speaker.log_info(
                     "Adding %s new connection(s) (max= %s, cur= %s)",
                     OKI_COL(dif), OKI_COL(max), OKI_COL(cur))
 
         for i in xrange(dif):
             p= PsygresFly(pg_conn)
-            q= PsygresPool.pool_queue
+            q= PsygresPool._pool_queue
             p._pool_queue= q
             q.put(p)
 
-        PsygresPool.pool_current+= dif
+        PsygresPool._pool_current+= dif
 
         return dif
 
+    ##
+    # Note that this function could be called by:
+    #  init_pool() -> no problem with mutexes
+    #  newSession() -> no problem with mutexes, coz
+    #                  lock is acquired.
+    ##
     @staticmethod
-    def addConnection(n=1, pg_conn= None):
-        m= PsygresPool.pool_current_mutex
+    def _addConnection(n=1, pg_conn= None):
+        """ 
+        Don't call me directly, use newSession() instead!
+        """
+        m= PsygresPool._pool_current_mutex
         m.acquire()
         PsygresPool._add_conn(n, pg_conn)
         m.release()
@@ -438,11 +477,12 @@ class PsygresPool(Speaker):
     @staticmethod
     def _init_pool(n= None, pg_conn= None):
 
-        n= n or PsygresPool.pool_min
+        n= n or PsygresPool._pool_min
 
         Speaker.log_info("Initializing connection Pool ...")
-        PsygresPool.pool_queue= PsygresPool.Queue()
-        PsygresPool.addConnection(n, pg_conn)
+        from Queue import Queue
+        PsygresPool._pool_queue= Queue()
+        PsygresPool._addConnection(n, pg_conn)
 
         PsygresPool._pool_initialized= True
 
@@ -450,18 +490,18 @@ class PsygresPool(Speaker):
     def _get_session():
         """ Never call me directly: use newSession!"""
 
-        db_pool= PsygresPool.pool_queue
+        db_pool= PsygresPool._pool_queue
 
         try:
             db= db_pool.get_nowait()
             return db
-        except PsygresPool.Empty:
+        except PsygresPool._empty_queue:
             pass
 
         if PsygresPool._full():
-            return db_pool.get(True, PsygresPool.pool_timeout)
+            return db_pool.get(True, PsygresPool._pool_timeout)
 
-        PsygresPool.addConnection()
+        PsygresPool._addConnection()
 
         return db_pool.get_nowait()
 
@@ -474,7 +514,7 @@ class PsygresPool(Speaker):
         if not PsygresPool._pool_initialized:
             return PsygresFly(pg_conn, name)
 
-        m= PsygresPool.pool_get_mutex
+        m= PsygresPool._pool_get_mutex
         m.acquire()
 
         try:
@@ -482,41 +522,52 @@ class PsygresPool(Speaker):
 
         except Exception, ss:
             m.release()
-            PsygresPool.log_int_raise(
+            Speaker.log_int_raise(
                     "Cannot retrieve Session from Pool (%s). FATAL.", 
                     ERR_COL(ss))
 
+        sname= name or PsygresFly._default_session
+
+        PsygresPool._add_orphan(sname, db)
+
         m.release()
+        db.session_name= sname 
 
-        db.session_name= name or PsygresFly._default_session
         return db
 
     @staticmethod
-    def back_home(pg_conn):
+    def backHome(pg_conn):
 
-        q= PsygresPool.pool_queue
-        m= PsygresPool.pool_current_mutex
+        q= PsygresPool._pool_queue
+        m= PsygresPool._pool_current_mutex
+        sname= pg_conn.session_name
 
         m.acquire()
 
         try:
             q.put_nowait(pg_conn)
         except Exception, ss:
-            PsygresPool.pool_current-= 1
-            PsygresPool.log_warning(
-                    "Removed connection from Pool: %s.", ERR_COL(ss))
+            PsygresPool._pool_current-= 1
+            Speaker.log_warning(
+                    "Removed connection %s from Pool: %s.", 
+                    ERR_COL(pg_conn.session_name), ERR_COL(ss))
+
+        PsygresPool._del_orphan(pg_conn)
 
         m.release()
 
+
     @staticmethod
-    def orphan():
+    def discard(pg_conn):
 
-        m= PsygresPool.pool_current_mutex
+        m= PsygresPool._pool_current_mutex
 
         m.acquire()
-        PsygresPool.pool_current-= 1
+        PsygresPool._pool_current-= 1
+        PsygresPool._del_orphan(pg_conn)
         m.release()
-        Speaker.log_warning("Connection dropped from Pool.")
+        Speaker.log_warning("Connection %s dropped from Pool.", 
+                ERR_COL(pg_conn.session_name))
 
 
 def init_dbda(conf, pg_conn= None):
@@ -549,17 +600,17 @@ def init_dbda(conf, pg_conn= None):
     pool= loc.get('DB_POOL', False)
 
     if pool:
-        pool_min= loc.get('DB_POOL_MIN', PsygresPool.pool_min)
-        pool_max= loc.get('DB_POOL_MAX', PsygresPool.pool_max)
-        pool_to= loc.get('DB_POOL_MIN', PsygresPool.pool_timeout)
+        pool_min= loc.get('DB_POOL_MIN', PsygresPool._pool_min)
+        pool_max= loc.get('DB_POOL_MAX', PsygresPool._pool_max)
+        pool_to= loc.get('DB_POOL_MIN', PsygresPool._pool_timeout)
 
         if pool_min> pool_max:
             Speaker.log_int_raise("Invalid configuration file: %s",
                     ERR_COL("DB_POOL_MIN> DB_POOL_MAX !!!"))
 
-        PsygresPool.pool_min= pool_min
-        PsygresPool.pool_max= pool_max
-        PsygresPool.pool_timeout= pool_to
+        PsygresPool._pool_min= pool_min
+        PsygresPool._pool_max= pool_max
+        PsygresPool._pool_timeout= pool_to
 
         PsygresPool._init_pool()
 
