@@ -33,8 +33,8 @@ MO_SAVE   = 3     # Save Action
 MO_LOAD   = 4     # Load Action
 MO_ULOAD  = 5     # Unsafe Load Action
 
-MO_BEFORE = 0
-MO_AFTER  = 1
+MO_BEFORE = 0     # Before?!?
+MO_AFTER  = 1     # After?!?
 
 #
 ## MoThEr: SQL symbols.
@@ -56,6 +56,18 @@ _A= ' AND '.join
 #
 
 def init_mother(cfile, pg_conn= None, fnaming= None):
+    """ init_mother(cfile [,pg_conn= None, fnaming= None]) -> None
+
+    Mother Initialization.
+    `cfile` is the Mother configuration file. You can obtain a 
+    default configuration file using mothermapper (option -C).
+
+    With pg_conn, you can pass your own psycopg connection. 
+    Don't do it, unless you have a very good reason.
+
+    `fnaming` is a function: auto created methods will be named
+    with this function. If None, a default function will be used.
+    """
 
     if not hasattr(DbMother, '_mo_initialized'):
         from speaker import Speaker
@@ -112,6 +124,13 @@ def init_mother(cfile, pg_conn= None, fnaming= None):
 
 
 def MotherSession(name= None, pg_conn= None):
+    """ MotherSession([name= None, pg_conn= None]) 
+
+    Returns a new Session. Each session is an isolated unit of work.
+    The queries made inside a session are automatically transaction'ed.
+    The new session has to be released, when his work is finished, with
+    session.endSession(). This call commit the queries and, if the Mother
+    Pool is used, put the connection inside the pool. """
 
     from dbda import PsygresPool
 
@@ -121,6 +140,21 @@ def MotherSession(name= None, pg_conn= None):
 
     return PsygresPool.newSession(name, pg_conn) 
 
+def MotherPoolStatus():
+    """MotherPoolStatus() -> (avail, total, min, max, active)
+
+    `avail` is the number of available connections.
+    `total` is the number of connection established.
+    Initializing Mother, `min` connections were made.
+    `max` connections are allowed on the Pool.
+    `active` is the number of used connection."""
+
+    from dbda import PsygresPool
+
+    if not PsygresPool._pool_initialized:
+        raise "No Pool Available: edit the Mother configuration file "\
+              "to use the Pool."
+    return PsygresPool.status()
 
 
 class _DbMap(Psygres):
@@ -134,7 +168,7 @@ class _DbMap(Psygres):
     def _load_map(map_file):
         from speaker import Speaker
         from os import path
-        if not path.isfile(map_file):
+        if not path.isfile(map_file, 'rb'):
             Speaker.log_int_raise("Mother Map %s does not exist.", ERR_COL(map_file)) 
         
         try:
@@ -225,13 +259,13 @@ class DbMother(_DbMap):
     _mo_fuck= " MoThEr fucker!"
     _mo_session_name= "MoSession-"
     _mo_flags= { 
-                MO_NOA: 'MO_NOA', 
-                MO_DEL: 'MO_DEL', 
-                MO_UP: 'MO_UP', 
-                MO_SAVE: 'MO_SAVE', 
-                MO_LOAD: 'MO_LOAD', 
-                MO_ULOAD: 'MO_ULOAD'
-                    }
+            MO_NOA: 'MO_NOA', 
+            MO_DEL: 'MO_DEL', 
+            MO_UP: 'MO_UP', 
+            MO_SAVE: 'MO_SAVE', 
+            MO_LOAD: 'MO_LOAD', 
+            MO_ULOAD: 'MO_ULOAD'
+            }
         
     # Mapping actions
     @staticmethod
@@ -692,7 +726,7 @@ class DbMother(_DbMap):
 
         onzo=self._invalidFields(fields)
         if onzo:
-            self.log_int_raise("Mth: cannot setFields(): "\
+            self.log_int_raise("Mth: cannot getFields(): "\
                     "invalid fields %s.", ERR_COL(str(onzo)))
         newd={}
         if autoload:
@@ -1505,11 +1539,11 @@ class MotherManager:
         mb= MotherBox(builder, qry_filter, MO_LOAD, fields, order, session= self.session)
         return mb
 
-    def relParams(self, related, fields= None):
-        """ relParams(self, related, fields) -> Mother Object
+    def relParams(self, related, fields= None, flag_obj= False):
+        """ relParams(self, related, [fields= None, flag_obj= False]) -> dict or Mother Object
 
-        Returns a Mother Object representing the relation record
-        between `self' and related records.
+        Returns, depending on flag_obj, a dict or  a Mother Object representing 
+        the relation record between `self' and related records.
         `related' is a list of related records (Mother Objects).
         if `fields' is not None, only these fields will be loaded."""
 
@@ -1524,7 +1558,8 @@ class MotherManager:
 
         r_obj.load(fields= fields)
 
-        return r_obj
+        if flag_obj: return r_obj
+        else: return r_obj.getFields()
 
     def initChildManager(self, children):
         """initChildManager(self,children) --> None
@@ -1677,10 +1712,10 @@ class MotherManager:
             return fly_join_rels
 
         def rel_params(builder):
-            def fly_rel_params(d, fields= None):
+            def fly_rel_params(d, fields= None, flag_obj= False):
                 if not isinstance(d, DbMother):
                     d= builder(d)
-                return self.relParams([d], fields)
+                return self.relParams([d], fields, flag_obj)
             return fly_rel_params
 
 
@@ -1739,11 +1774,11 @@ class MotherManager:
             if not hasattr(self, attr_name):
                 attr= rel_params(rel)
                 attr.__doc__=\
-                    " %s(obj [,fields= None]) --> Mother Object\n\n"                \
-                    "Returns a Mother Object that represents the relation\n"        \
-                    "between `self' and the record on the table |%s|, which is\n"   \
-                    "represented by obj (a dict or a Mother Object).\n"             \
-                    "If `fields' is specified, load only these fields."             \
+                    " %s(obj [,fields= None, flag_obj= False]) --> dict Mother Object\n\n"  \
+                    "Returns, depending on flag_obj,  a dict or a Mother Object that\n"     \
+                    "represents the relation between `self' and the record on the table\n"  \
+                    "|%s|, which is represented by obj (a dict or a Mother Object).\n"      \
+                    "If `fields' is specified, load only these fields."                     \
                     % (attr_name, rtbl)
 
                 setattr(self, attr_name, attr)
